mod Maux logic: 'cl'

incl Mtex

incl Mtexbt

rem 
  \para \bf  Numeric Functions.  \end 

rem 
  \para \it  Maximum.  \end 

fun/2 Max 'Tex_f2_n_max'
  Max(x,y) = x <- x >= y
  Max(x,y) = y <- x < y

rem 
  \para \it  Exponentiation.  \end 

fun/2 Exp 'Tex_f2_n_exp'
  Exp(x,0) = 1
  Exp(x,y+1) = x*Exp(x,y)

thm Exp_is_zero
  Exp(x,y) = 0 <-> x = 0 & y != 0
proof 
 ind N; y @ 0; y
  proved
  weak Exp(x,y) = 0 <-> x = 0 & y != 0
   cut x = 0
    proved
    proved....

thm Exp_is_one
  Exp(x,y) = 1 <-> x = 1 \/ y = 0
proof 
 ind N; y @ 0; y
  proved
  split* x = 1 & (x = 1 \/ y = 0) <-> x = 1.2,0,0
   proved
   proved...

thm Exp_plus
  Exp(x,y+z) = Exp(x,y)*Exp(x,z)
proof 
 ind N; y @ 0; y
  proved
  proved..

thm Exp_minus
  x != 0 & y >= z -> Exp(x,y-z) = Exp(x,y)/Exp(x,z)
proof 
 ind N; y; z @ 0; y
  proved
  case N; z @ 0; z1
   proved
   inst \a z(y >= z -> Exp(x,y-z) = Exp(x,y)/Exp(x,z)); z1 proved....

thm Exp_cancel_eq
  x > 1 -> Exp(x,y) = Exp(x,z) <-> y = z
proof 
 ind N; y; z @ 0; y
  use Exp_is_one.0; x; z proved.
  case N; z @ 0; z1
   proved
   inst \a z(Exp(x,y) = Exp(x,z) <-> y = z).0; z1 proved....

thm Exp_cancel_le
  x > 1 -> Exp(x,y) <= Exp(x,z) <-> y <= z
proof 
 ind N; y; z @ 0; y
  use Exp_is_zero.0; x; z proved.
  case N; z @ 0; z1
   use Exp_is_zero.0; x; y proved.
   inst \a z(Exp(x,y) <= Exp(x,z) <-> y <= z).0; z1 proved....

thm Exp_cancel_lt
  x > 1 -> Exp(x,y) < Exp(x,z) <-> y < z
proof 
 ind N; y; z @ 0; y
  cut z = 0
   proved
   use Exp_is_zero.0; x; z-1 proved..
  case N; z @ 0; z1
   use Exp_is_zero.0; x; y proved.
   inst \a z(Exp(x,y) < Exp(x,z) <-> y < z).0; z1 proved....

rem 
  \para \bf  Lists.  \end 

rem 
  \para \it  List length.  \end 

fun L 
  L(0) = 0
  L(v,w) = L(w)+1

thm L_is_zero
  L(x) = 0 <-> x = 0
proof 
 case Ln; x @ 0; v,w
  proved
  proved..

thm L_conc
  L(x++y) = L(x)+L(y)
proof 
 ind Ln; x @ 0; v,w
  proved
  proved..

rem 
  \para \it  List indexing.  \end 

fun/2 Sub 'Tex_f2_ln_sub'
  Sub((v,w),0) = v
  Sub((v,w),i+1) = Sub(w,i)

rem 
  \para \it  Half-open interval.  \end 

fun/2 Interval 'Tex_f2_ln_interval_co'
  Interval(m,n) = 0 <- m >= n
  Interval(m,n) = m,Interval(m+1,n) <- m < n

thm Interval_l
  L Interval(m,n) = n-m
proof 
 indm n-m @ n1,m1
  case Dich; n,m @ x,y; x,y
   proved
   inst \a n1\a m1(m+(n1-m1) < n -> L Interval(m1,n1) = n1-m1); n; m+1 proved....

thm Interval_sub
  m+i < n -> Sub(Interval(m,n),i) = m+i
proof 
 indm n-m; i @ n1,m1
  case Dich; n,m+1 @ x,y; x,y
   proved
   case N; i @ 0; i1
    proved
    inst \a n1\a m1(m+(n1-m1) < n -> 
                    \a i(i+m1 < n1 -> Sub((m1,Interval(m1+1,n1)),i) = i+m1)):0,2,
                                                                             1,0; n; m+1; i1 proved.....

thm Interval_conc
  k <= m & m <= n -> Interval(k,m)++Interval(m,n) = Interval(k,n)
proof 
 indm m-k @ m1,k1
  case Dich; m,k @ x,y; x,y
   proved
   inst \a m1\a k1(k+(m1-k1) < m -> k1 <= m1 & m1 <= n -> 
                   Interval(k1,m1)++Interval(m1,n) = Interval(k1,n)); m; k+1 proved....

rem 
  \para \bf  Binary Trees.  \end 

rem 
  \para \it  Arithmetization.  \end 

fun/0 E 'Tex_f0_bt_e'
  E = 0,0

fun/3 Nd 'Tex_f3_bt_nd'
  Nd(x,l,r) = 1,x,l,r

pred Bt 
  Bt(E)
  Bt Nd(x,l,r) <- N(x) & Bt(l) & Bt(r)

rem 
  \para \it  Size and depth of binary trees.  \end 

fun Sz 'Tex_f1_bt_sz'
  Sz(E) = 0
  Sz Nd(x,l,r) = Sz(l)+Sz(r)+1

fun Dp 'Tex_f1_bt_dp'
  Dp(E) = 0
  Dp Nd(x,l,r) = Max(Dp(l),Dp(r))+1

thm Sz_is_zero
  Bt(t) -> Sz(t) = 0 <-> t = E
proof 
 case Bt; t @ 0; x,l,r
  proved
  proved..

thm Dp_is_zero
  Bt(t) -> Dp(t) = 0 <-> t = E
proof 
 case Bt; t @ 0; x,l,r
  proved
  proved..

thm Dp_sz
  Bt(t) -> Dp(t) <= Sz(t) & Sz(t) < Exp(2,Dp(t))
proof 
 ind Bt; t @ 0; x,l,r
  proved
  case Dich; Dp(r),Dp(l) @ x1,y; x1,y
   use Exp_cancel_le.0; 2; Dp(r); Dp(l) proved.
   use Exp_cancel_le.0; 2; Dp(l); Dp(r) proved....

rem 
  \para \it  Membership in binary trees.  \end 

pred/2 Inbt 'Tex_p2_bt_in'
  Inbt(x,Nd(y,l,r)) <- x = y
  Inbt(x,Nd(y,l,r)) <- x != y & Inbt(x,l)
  Inbt(x,Nd(y,l,r)) <- x != y & ~Inbt(x,l) & Inbt(x,r)

thm Inbt_e
  ~Inbt(x,E)
proof  proved.

thm Inbt_nd
  Inbt(x,Nd(y,l,r)) <-> x = y \/ Inbt(x,l) \/ Inbt(x,r)
proof 
 cut x = y
  proved
  cut Inbt(x,l)
   proved
   proved...

rem 
  \para \it  Isomorphic binary trees.  \end 

pred/2 Isomorphic 'Tex_p2_bt_isomorphic'
  Isomorphic(E,E)
  Isomorphic(Nd(x1,l1,r1),Nd(x2,l2,r2)) <- Isomorphic(l1,l2) & Isomorphic(r1,r2)

thm Isomorphic_refl
  Bt(t) -> Isomorphic(t,t)
proof 
 ind Bt; t @ 0; x,l,r
  proved
  proved..

thm Isomorphic_sym
  Isomorphic(t1,t2) -> Isomorphic(t2,t1)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

thm Isomorphic_trans
  Isomorphic(t1,t2) & Isomorphic(t2,t3) -> Isomorphic(t1,t3)
proof 
 ind Isomorphic; t1,t2; t3 @ 0; x1,l1,r1,x2,l2,r2
  proved
  case Isomorphic; Nd(x2,l2,r2),t3 @ 0; x22,l22,r22,x3,l3,r3
   proved
   inst \a t3(Isomorphic(l22,t3) -> Isomorphic(l1,t3)); l3
    inst \a t3(Isomorphic(r22,t3) -> Isomorphic(r1,t3)); r3 proved.....

thm Isomorphic_bt
  Bt(t1) & Isomorphic(t1,t2) -> Bt(t2)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

thm Isomorphic_sz
  Isomorphic(t1,t2) -> Sz(t1) = Sz(t2)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

thm Isomorphic_dp
  Isomorphic(t1,t2) -> Dp(t1) = Dp(t2)
proof 
 ind Isomorphic; t1,t2 @ 0; x1,l1,r1,x2,l2,r2
  proved
  proved..

rem 
  \para \it  Traversals of binary trees.  \end 

fun Preorder 
  Preorder(E) = 0
  Preorder Nd(x,l,r) = x,Preorder(l)++Preorder(r)

fun Inorder 
  Inorder(E) = 0
  Inorder Nd(x,l,r) = Inorder(l)++(x,Inorder(r))

fun Postorder 
  Postorder(E) = 0
  Postorder Nd(x,l,r) = Postorder(l)++Postorder(r)++(x,0)

